import { IncomingForm } from 'formidable';
import { checkAuth } from '../middleware/auth.ts';
import { router, clients, writeEvents } from '../router.ts';
import { error } from '../middleware/auth.ts';
import fs from 'fs';
import { useFileStore } from '@kevisual/use-config/file-store';
import { app, minioClient } from '@/app.ts';
import { bucketName } from '@/modules/minio.ts';
import { getContentType } from '@/utils/get-content-type.ts';
import { hash } from 'crypto';
import { MicroAppUploadModel } from '@/routes/micro-app/models.ts';
const cacheFilePath = useFileStore('cache-file', { needExists: true });

router.post('/api/micro-app/upload', async (req, res) => {
  if (res.headersSent) return; // 如果响应已发送，不再处理
  res.writeHead(200, { 'Content-Type': 'application/json' });
  const { tokenUser, token } = await checkAuth(req, res);
  if (!tokenUser) return;
  //
  // 使用 formidable 解析 multipart/form-data
  const form = new IncomingForm({
    multiples: false, // 支持多文件上传
    uploadDir: cacheFilePath, // 上传文件存储目录
    allowEmptyFiles: true, // 允许空
    minFileSize: 0, // 最小文件大小
    maxFiles: 1, // 最大文件数量
    createDirsFromUploads: false, // 根据上传的文件夹结构创建目录
    keepExtensions: true, // 保留文件
    hashAlgorithm: 'md5', // 文件哈希算法
  });
  form.on('progress', (bytesReceived, bytesExpected) => {
    const progress = (bytesReceived / bytesExpected) * 100;
    console.log(`Upload progress: ${progress.toFixed(2)}%`);
    const data = {
      progress: progress.toFixed(2),
      message: `Upload progress: ${progress.toFixed(2)}%`,
    };
    writeEvents(req, data);
  });
  // 解析上传的文件
  form.parse(req, async (err, fields, files) => {
    if (err) {
      res.end(error(`Upload error: ${err.message}`));
      const uploadedFiles = Array.isArray(files.file) ? files.file : [files.file];
      uploadedFiles.forEach((file) => {
        fs.unlinkSync(file.filepath);
      });
      return;
    }
    const clearFiles = () => {
      const uploadedFiles = Array.isArray(files.file) ? files.file : [files.file];
      uploadedFiles.forEach((file) => {
        fs.unlinkSync(file.filepath);
      });
    };
    let appKey, collection;
    const { appKey: _appKey, collection: _collecion } = fields;
    if (Array.isArray(_appKey)) {
      appKey = _appKey?.[0];
    } else {
      appKey = _appKey;
    }
    if (Array.isArray(_collecion)) {
      collection = _collecion?.[0];
    } else {
      collection = _collecion;
    }
    collection = parseIfJson(collection);

    appKey = appKey || 'micro-app';
    console.log('Appkey', appKey);
    console.log('collection', collection);
    // 逐个处理每个上传的文件
    const uploadedFiles = Array.isArray(files.file) ? files.file : [files.file];
    const uploadResults = [];
    for (let i = 0; i < uploadedFiles.length; i++) {
      const file = uploadedFiles[i];
      // @ts-ignore
      const tempPath = file.filepath; // 文件上传时的临时路径
      const relativePath = file.originalFilename; // 保留表单中上传的文件名 (包含文件夹结构)
      // 比如 child2/b.txt
      const minioPath = `private/${tokenUser.username}/${appKey}/${relativePath}`;
      // 上传到 MinIO 并保留文件夹结构
      const isHTML = relativePath.endsWith('.html');
      await minioClient.fPutObject(bucketName, minioPath, tempPath, {
        'Content-Type': getContentType(relativePath),
        'app-source': 'user-micro-app',
        'Cache-Control': isHTML ? 'no-cache' : 'max-age=31536000, immutable', // 缓存一年
      });
      uploadResults.push({
        name: relativePath,
        path: minioPath,
        hash: file.hash,
        size: file.size,
      });
      fs.unlinkSync(tempPath); // 删除临时文件
    }
    // 受控
    const r = await app.call({
      path: 'micro-app',
      key: 'upload',
      payload: {
        token: token,
        data: {
          appKey,
          collection,
          files: uploadResults,
        },
      },
    });
    const data: any = {
      code: r.code,
      data: r.body,
    };
    if (r.message) {
      data.message = r.message;
    }
    res.end(JSON.stringify(data));
  });
});

router.get('/api/micro-app/download/:id', async (req, res) => {
  const { id } = req.params;

  if (!id) {
    res.writeHead(200, { 'Content-Type': 'application/javascript; charset=utf-8' });
    res.end(error('Key parameter is required'));
    return;
  }
  const query = new URL(req.url || '', 'http://localhost');
  const notNeedToken = query.searchParams.get('notNeedToken') || '';
  const fileTitle = query.searchParams.get('title') || '';
  if (res.headersSent) return; // 如果响应已发送，不再处理
  let tokenUser;
  if (!DEV_SERVER && !notNeedToken) {
    const auth = await checkAuth(req, res);
    tokenUser = auth.tokenUser;
    if (!tokenUser) return;
  }
  let file: MicroAppUploadModel | null = null;
  if (!DEV_SERVER) {
    // file.uid !== tokenUser.id && res.end(error('No permission', 403));
    // return;
  }
  if (fileTitle) {
    file = await MicroAppUploadModel.findOne({
      where: { title: fileTitle },
    });
  } else if (id) {
    file = await MicroAppUploadModel.findByPk(id);
  }

  if (!file) {
    res.end(error('File not found'));
    return;
  }
  const objectName = file.data?.file?.path;
  const fileName = file.data?.file?.name;
  if (!objectName) {
    res.end(error('File not found'));
    return;
  }
  res.setHeader('Content-Disposition', `attachment; filename="${encodeURIComponent(fileName)}"`);
  res.setHeader('app-key', file.data?.key || id);
  res.writeHead(200, { 'Content-Type': 'application/octet-stream' });
  try {
    const stream = await minioClient.getObject(bucketName, objectName);
    // 捕获流的错误，防止崩溃
    stream.on('error', (err) => {
      console.error('Error while streaming file:', err);
      if (!res.headersSent) {
        res.end(error('Error downloading file'));
      }
    });

    stream.pipe(res).on('finish', () => {
      console.log(`File download completed: ${id}`);
    });
  } catch (err) {
    console.error('Error during download:', err);
    if (!res.headersSent) {
      res.end(error('Error downloading file'));
    }
  }
});
function parseIfJson(collection: any): any {
  try {
    return JSON.parse(collection);
  } catch (e) {
    return collection;
  }
}
